@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #ifndef _TBB_scheduler_H
@@ -44,6 +36,7 @@ namespace tbb {
 namespace internal {
 
 template<typename SchedulerTraits> class custom_scheduler;
+struct nested_arena_context;
 
 //------------------------------------------------------------------------
 // generic_scheduler
@@ -61,18 +54,9 @@ struct scheduler_list_node_t {
 
 #define LockedMaster ((generic_scheduler*)~(intptr_t)0)
 
-class governor;
-class market;
-class arena;
-
-#if __TBB_SCHEDULER_OBSERVER
-class task_scheduler_observer_v3;
-class observer_proxy;
-#endif /* __TBB_SCHEDULER_OBSERVER */
-
 struct scheduler_state {
     //! Index of the arena slot the scheduler occupies now, or occupied last time.
-    size_t my_arena_index;
+    size_t my_arena_index; // TODO: make it unsigned and pair with my_affinity_id to fit into cache line
 
     //! Pointer to the slot in the arena we own at the moment.
     arena_slot* my_arena_slot;
@@ -105,6 +89,15 @@ struct scheduler_state {
     //! Last observer in the local observers list processed by this scheduler
     observer_proxy* my_last_local_observer;
 #endif /* __TBB_SCHEDULER_OBSERVER */
+#if __TBB_TASK_PRIORITY
+    //! Latest known highest priority of tasks in the market or arena.
+    /** Master threads currently tracks only tasks in their arenas, while workers
+        take into account global top priority (among all arenas in the market). **/
+    volatile intptr_t *my_ref_top_priority;
+
+    //! Pointer to market's (for workers) or current arena's (for the master) reload epoch counter.
+    volatile uintptr_t *my_ref_reload_epoch;
+#endif /* __TBB_TASK_PRIORITY */
 };
 
 //! Work stealing task scheduler.
@@ -114,27 +107,8 @@ struct scheduler_state {
     Class generic_scheduler is an abstract base class that contains most of the scheduler,
     except for tweaks specific to processors and tools (e.g. VTune).
     The derived template class custom_scheduler<SchedulerTraits> fills in the tweaks. */
-class generic_scheduler: public scheduler, public ::rml::job, private scheduler_state {
-    friend class tbb::task;
-    friend class market;
-    friend class arena;
-#if __TBB_TASK_ARENA
-    friend class interface6::task_arena;
-    friend class interface6::delegated_task;
-    friend class interface6::wait_task;
-    friend struct interface6::wait_body;
-#endif //__TBB_TASK_ARENA
-    friend class allocate_root_proxy;
-    friend class governor;
-#if __TBB_TASK_GROUP_CONTEXT
-    friend class allocate_root_with_context_proxy;
-    friend class tbb::task_group_context;
-#endif /* __TBB_TASK_GROUP_CONTEXT */
-#if __TBB_SCHEDULER_OBSERVER
-    friend class task_scheduler_observer_v3;
-#endif /* __TBB_SCHEDULER_OBSERVER */
-    friend class scheduler;
-    template<typename SchedulerTraits> friend class custom_scheduler;
+class generic_scheduler: public scheduler, public ::rml::job, public scheduler_state {
+public: // almost every class in TBB uses generic_scheduler
 
     //! If sizeof(task) is <=quick_task_size, it is handled on a free list instead of malloc'd.
     static const size_t quick_task_size = 256-task_prefix_reservation_size;
@@ -167,10 +141,6 @@ class generic_scheduler: public scheduler, public ::rml::job, private scheduler_
     //! Random number generator used for picking a random victim from which to steal.
     FastRandom my_random;
 
-    //! Hint provided for operations with the container of starvation-resistant tasks.
-    /** Modified by the owner thread (during these operations). **/
-    unsigned hint_for_push; //TODO: Replace by my_random?
-
     //! Free list of small tasks that can be reused.
     task* my_free_list;
 
@@ -277,11 +247,6 @@ class generic_scheduler: public scheduler, public ::rml::job, private scheduler_
         return t.prefix().extra_state==es_task_proxy;
     }
 
-    //! Get a task from the starvation-resistant task stream of the current arena.
-    /** Returns the pointer to the task, or NULL if the attempt was unsuccessful. 
-        The latter case does not mean that the stream is drained, however. **/
-    task* dequeue_task();
-
     //! Steal task from another scheduler's ready pool.
     task* steal_task( arena_slot& victim_arena_slot );
 
@@ -307,8 +272,10 @@ class generic_scheduler: public scheduler, public ::rml::job, private scheduler_
     static void cleanup_worker( void* arg, bool worker );
 
 protected:
+    template<typename SchedulerTraits> friend class custom_scheduler;
     generic_scheduler( arena*, size_t index );
 
+public:
 #if TBB_USE_ASSERT > 1
     //! Check that internal data structures are in consistent state.
     /** Raises __TBB_ASSERT failure if inconsistency is found. */
@@ -317,10 +284,10 @@ protected:
     void assert_task_pool_valid() const {}
 #endif /* TBB_USE_ASSERT <= 1 */
 
-public:
 #if __TBB_TASK_ARENA
-    template<typename Body>
-    void nested_arena_execute(arena*, task*, bool, Body&);
+    void nested_arena_entry(arena*, nested_arena_context &, bool);
+    void nested_arena_exit(nested_arena_context &);
+    void wait_until_empty();
 #endif
 
     /*override*/ 
@@ -385,15 +352,14 @@ public:
     static task* plugged_return_list() {return (task*)(intptr_t)(-1);}
 
     //! Number of small tasks that have been allocated by this scheduler. 
-    intptr_t my_small_task_count;
+    __TBB_atomic intptr_t my_small_task_count;
 
     //! List of small tasks that have been returned to this scheduler by other schedulers.
     task* my_return_list;
 
     //! Try getting a task from other threads (via mailbox, stealing, FIFO queue, orphans adoption).
     /** Returns obtained task or NULL if all attempts fail. */
-    virtual task* receive_or_steal_task( __TBB_atomic reference_count& completion_ref_count,
-                                         bool return_if_no_work ) = 0;
+    virtual task* receive_or_steal_task( __TBB_atomic reference_count& completion_ref_count ) = 0;
 
     //! Free a small task t that that was allocated by a different scheduler 
     void free_nonlocal_small_task( task& t ); 
@@ -428,11 +394,6 @@ public:
     //! Returns reference priority used to decide whether a task should be offloaded.
     inline intptr_t effective_reference_priority () const;
 
-    //! Latest known highest priority of tasks in the market or arena.
-    /** Master threads currently tracks only tasks in their arenas, while workers
-        take into account global top priority (among all arenas in the market). **/
-    volatile intptr_t *my_ref_top_priority;
-
     // TODO: move into slots and fix is_out_of_work
     //! Task pool for offloading tasks with priorities lower than the current top priority.
     task* my_offloaded_tasks;
@@ -440,9 +401,6 @@ public:
     //! Points to the last offloaded task in the my_offloaded_tasks list.
     task** my_offloaded_task_list_tail_link;
 
-    //! Pointer to market's (for workers) or current arena's (for the master) reload epoch counter.
-    volatile uintptr_t *my_ref_reload_epoch;
-
     //! Indicator of how recently the offload area was checked for the presence of top priority tasks.
     uintptr_t my_local_reload_epoch;
 
@@ -471,12 +429,41 @@ public:
     //! and propagates the new state to them.
     template <typename T>
     void propagate_task_group_state ( T task_group_context::*mptr_state, task_group_context& src, T new_state );
+
+    // check consistency
+    static void assert_context_valid(const task_group_context *tgc) {
+        suppress_unused_warning(tgc);
+#if TBB_USE_ASSERT
+        __TBB_ASSERT(tgc, NULL);
+        uintptr_t ctx = tgc->my_version_and_traits;
+        __TBB_ASSERT(is_alive(ctx), "referenced task_group_context was destroyed");
+        static const char *msg = "task_group_context is invalid";
+        __TBB_ASSERT(!(ctx&~(3|(7<<task_group_context::traits_offset))), msg); // the value fits known values of versions and traits
+        __TBB_ASSERT(tgc->my_kind < task_group_context::dying, msg);
+        __TBB_ASSERT(tgc->my_cancellation_requested == 0 || tgc->my_cancellation_requested == 1, msg);
+        __TBB_ASSERT(tgc->my_state < task_group_context::low_unused_state_bit, msg);
+        if(tgc->my_kind != task_group_context::isolated) {
+            __TBB_ASSERT(tgc->my_owner, msg);
+            __TBB_ASSERT(tgc->my_node.my_next && tgc->my_node.my_prev, msg);
+        }
+#if __TBB_TASK_PRIORITY
+        assert_priority_valid(tgc->my_priority);
+#endif
+        if(tgc->my_parent)
+#if TBB_USE_ASSERT > 1
+            assert_context_valid(tgc->my_parent);
+#else
+            __TBB_ASSERT(is_alive(tgc->my_parent->my_version_and_traits), msg);
+#endif
+#endif
+    }
 #endif /* __TBB_TASK_GROUP_CONTEXT */
 
 #if _WIN32||_WIN64
 private:
     //! Handle returned by RML when registering a master with RML
     ::rml::server::execution_resource_t master_exec_resource;
+public:
 #endif /* _WIN32||_WIN64 */
 
 #if __TBB_TASK_GROUP_CONTEXT
@@ -619,8 +606,7 @@ void generic_scheduler::commit_relocated_tasks ( size_t new_tail ) {
 template<free_task_hint hint>
 void generic_scheduler::free_task( task& t ) {
 #if __TBB_HOARD_NONLOCAL_TASKS
-    // TODO: remove the whole free_task_hint stuff when enabled permanently
-    static const free_task_hint h = no_hint;
+    static const int h = hint&(~local_task);
 #else
     static const free_task_hint h = hint;
 #endif
@@ -629,6 +615,7 @@ void generic_scheduler::free_task( task& t ) {
     // Verify that optimization hints are correct.
     __TBB_ASSERT( h!=small_local_task || p.origin==this, NULL );
     __TBB_ASSERT( !(h&small_task) || p.origin, NULL );
+    __TBB_ASSERT( !(h&local_task) || (!p.origin || uintptr_t(p.origin) > uintptr_t(4096)), "local_task means allocated");
     poison_value(p.depth);
     poison_value(p.ref_count);
     poison_pointer(p.owner);
@@ -638,17 +625,18 @@ void generic_scheduler::free_task( task& t ) {
         GATHER_STATISTIC(++my_counters.free_list_length);
         p.next = my_free_list;
         my_free_list = &t;
-    } else if( p.origin && uintptr_t(p.origin) < uintptr_t(4096) ) {
+    } else if( !(h&local_task) && p.origin && uintptr_t(p.origin) < uintptr_t(4096) ) {
         // a special value reserved for future use, do nothing since
         // origin is not pointing to a scheduler instance
     } else if( !(h&local_task) && p.origin ) {
         GATHER_STATISTIC(++my_counters.free_list_length);
 #if __TBB_HOARD_NONLOCAL_TASKS
-        p.next = my_nonlocal_free_list;
-        my_nonlocal_free_list = &t;
-#else
-        free_nonlocal_small_task(t);
+        if( !(h&no_cache) ) {
+            p.next = my_nonlocal_free_list;
+            my_nonlocal_free_list = &t;
+        } else
 #endif
+        free_nonlocal_small_task(t);
     } else {
         GATHER_STATISTIC(--my_counters.big_tasks);
         deallocate_task(t);
@@ -679,6 +667,41 @@ inline void generic_scheduler::offload_task ( task& t, intptr_t /*priority*/ ) {
 }
 #endif /* __TBB_TASK_PRIORITY */
 
+#if __TBB_FP_CONTEXT
+class cpu_ctl_env_helper {
+    cpu_ctl_env guard_cpu_ctl_env;
+    cpu_ctl_env curr_cpu_ctl_env;
+public:
+    cpu_ctl_env_helper() {
+        guard_cpu_ctl_env.get_env();
+        curr_cpu_ctl_env = guard_cpu_ctl_env;
+    }
+    ~cpu_ctl_env_helper() {
+        if ( curr_cpu_ctl_env != guard_cpu_ctl_env )
+            guard_cpu_ctl_env.set_env();
+    }
+    void set_env( const task_group_context *ctx ) {
+        generic_scheduler::assert_context_valid(ctx);
+        const cpu_ctl_env &ctl = *punned_cast<cpu_ctl_env*>(&ctx->my_cpu_ctl_env);
+        if ( ctl != curr_cpu_ctl_env ) {
+            curr_cpu_ctl_env = ctl;
+            curr_cpu_ctl_env.set_env();
+        }
+    }
+    void restore_default() {
+        if ( curr_cpu_ctl_env != guard_cpu_ctl_env ) {
+            guard_cpu_ctl_env.set_env();
+            curr_cpu_ctl_env = guard_cpu_ctl_env;
+        }
+    }
+};
+#else
+struct cpu_ctl_env_helper {
+    void set_env( __TBB_CONTEXT_ARG1(task_group_context *) ) {}
+    void restore_default() {}
+};
+#endif /* __TBB_FP_CONTEXT */
+
 } // namespace internal
 } // namespace tbb
 
