@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #ifndef _TBB_malloc_Customize_H_
@@ -45,11 +37,13 @@
 #define MALLOC_ITT_SYNC_ACQUIRED(pointer) ITT_NOTIFY(sync_acquired, (pointer))
 #define MALLOC_ITT_SYNC_RELEASING(pointer) ITT_NOTIFY(sync_releasing, (pointer))
 #define MALLOC_ITT_SYNC_CANCEL(pointer) ITT_NOTIFY(sync_cancel, (pointer))
+#define MALLOC_ITT_FINI_ITTLIB()        ITT_FINI_ITTLIB()
 #else
 #define MALLOC_ITT_SYNC_PREPARE(pointer) ((void)0)
 #define MALLOC_ITT_SYNC_ACQUIRED(pointer) ((void)0)
 #define MALLOC_ITT_SYNC_RELEASING(pointer) ((void)0)
 #define MALLOC_ITT_SYNC_CANCEL(pointer) ((void)0)
+#define MALLOC_ITT_FINI_ITTLIB()        ((void)0)
 #endif
 
 //! Stripped down version of spin_mutex.
@@ -147,12 +141,15 @@ static inline bool isPowerOfTwoMultiple(uintptr_t arg, uintptr_t divisor) {
 
 #define USE_DEFAULT_MEMORY_MAPPING 1
 
-// To support malloc replacement with LD_PRELOAD
+// To support malloc replacement
 #include "proxy.h"
 
 #if MALLOC_UNIXLIKE_OVERLOAD_ENABLED
 #define malloc_proxy __TBB_malloc_proxy
 extern "C" void * __TBB_malloc_proxy(size_t)  __attribute__ ((weak));
+#elif MALLOC_ZONE_OVERLOAD_ENABLED
+// as there is no significant overhead, always suppose that proxy can be present
+const bool malloc_proxy = true;
 #else
 const bool malloc_proxy = false;
 #endif
@@ -164,4 +161,64 @@ namespace internal {
 
 #define MALLOC_EXTRA_INITIALIZATION rml::internal::init_tbbmalloc()
 
+// Need these to work regardless of tools support.
+namespace tbb {
+    namespace internal {
+
+        enum notify_type {prepare=0, cancel, acquired, releasing};
+
+#if TBB_USE_THREADING_TOOLS
+        inline void call_itt_notify(notify_type t, void *ptr) {
+            switch ( t ) {
+            case prepare:
+                MALLOC_ITT_SYNC_PREPARE( ptr );
+                break;
+            case cancel:
+                MALLOC_ITT_SYNC_CANCEL( ptr );
+                break;
+            case acquired:
+                MALLOC_ITT_SYNC_ACQUIRED( ptr );
+                break;
+            case releasing:
+                MALLOC_ITT_SYNC_RELEASING( ptr );
+                break;
+            }
+        }
+#else
+        inline void call_itt_notify(notify_type /*t*/, void * /*ptr*/) {}
+#endif // TBB_USE_THREADING_TOOLS
+
+        template <typename T>
+        inline void itt_store_word_with_release(T& dst, T src) {
+#if TBB_USE_THREADING_TOOLS
+            call_itt_notify(releasing, &dst);
+#endif // TBB_USE_THREADING_TOOLS
+            FencedStore(*(intptr_t*)&dst, src); 
+        }
+
+        template <typename T>
+        inline T itt_load_word_with_acquire(T& src) {
+            T result = FencedLoad(*(intptr_t*)&src);
+#if TBB_USE_THREADING_TOOLS
+            call_itt_notify(acquired, &src);
+#endif // TBB_USE_THREADING_TOOLS
+            return result;
+
+        }
+    } // namespace internal
+} // namespace tbb
+
+#include "tbb/internal/_aggregator_impl.h"
+
+template <typename OperationType>
+struct MallocAggregator {
+    typedef tbb::internal::aggregator_generic<OperationType> type;
+};
+
+//! aggregated_operation base class
+template <typename Derived>
+struct MallocAggregatedOperation {
+    typedef tbb::internal::aggregated_operation<Derived> type;
+};
+
 #endif /* _TBB_malloc_Customize_H_ */
