@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #include "proxy.h"
@@ -39,12 +31,16 @@
     #define TBB_USE_EXCEPTIONS 1
 #endif
 
-#if MALLOC_UNIXLIKE_OVERLOAD_ENABLED
+#if MALLOC_UNIXLIKE_OVERLOAD_ENABLED || MALLOC_ZONE_OVERLOAD_ENABLED
+
+#ifndef __THROW
+#define __THROW
+#endif
 
 /*** service functions and variables ***/
 
+#include <string.h> // for memset
 #include <unistd.h> // for sysconf
-#include <dlfcn.h>
 
 static long memoryPageSize;
 
@@ -53,83 +49,133 @@ static inline void initPageSize()
     memoryPageSize = sysconf(_SC_PAGESIZE);
 }
 
-/* For the expected behaviour (i.e., finding malloc/free/etc from libc.so,
-   not from ld-linux.so) dlsym(RTLD_NEXT) should be called from
-   a LD_PRELOADed library, not another dynamic library.
-   So we have to put find_original_malloc here.
- */
-extern "C" bool __TBB_internal_find_original_malloc(int num, const char *names[],
-                                                        void *ptrs[])
-{
-    for (int i=0; i<num; i++)
-        if (NULL == (ptrs[i] = dlsym (RTLD_NEXT, names[i])))
-            return false;
-
-    return true;
-}
+#if MALLOC_UNIXLIKE_OVERLOAD_ENABLED
+#include "Customize.h" // FencedStore
+#include <dlfcn.h>
+#include <malloc.h>    // mallinfo
 
 /* __TBB_malloc_proxy used as a weak symbol by libtbbmalloc for:
    1) detection that the proxy library is loaded
    2) check that dlsym("malloc") found something different from our replacement malloc
 */
-extern "C" void *__TBB_malloc_proxy() __attribute__ ((alias ("malloc")));
+extern "C" void *__TBB_malloc_proxy(size_t) __attribute__ ((alias ("malloc")));
 
-#ifndef __THROW
-#define __THROW
-#endif
+static void *orig_msize;
+
+#elif MALLOC_ZONE_OVERLOAD_ENABLED
+
+#include "proxy_overload_osx.h"
+
+#endif // MALLOC_ZONE_OVERLOAD_ENABLED
+
+// Original (i.e., replaced) functions,
+// they are never changed for MALLOC_ZONE_OVERLOAD_ENABLED.
+static void *orig_free,
+    *orig_realloc;
+
+#if MALLOC_UNIXLIKE_OVERLOAD_ENABLED
+#define ZONE_ARG
+#define PREFIX(name) name
+
+static void *orig_libc_free,
+    *orig_libc_realloc;
+
+// We already tried to find ptr to original functions.
+static intptr_t origFuncSearched;
+
+inline void InitOrigPointers()
+{
+    // race is OK here, as different threads found same functions
+    if (!origFuncSearched) {
+        orig_free = dlsym(RTLD_NEXT, "free");
+        orig_realloc = dlsym(RTLD_NEXT, "realloc");
+        orig_msize = dlsym(RTLD_NEXT, "malloc_usable_size");
+        orig_libc_free = dlsym(RTLD_NEXT, "__libc_free");
+        orig_libc_realloc = dlsym(RTLD_NEXT, "__libc_realloc");
+
+        FencedStore(origFuncSearched, 1);
+    }
+}
 
 /*** replacements for malloc and the family ***/
-
 extern "C" {
+#elif MALLOC_ZONE_OVERLOAD_ENABLED
 
-void *malloc(size_t size) __THROW
+// each impl_* function has such 1st argument, it's unused
+#define ZONE_ARG struct _malloc_zone_t *,
+#define PREFIX(name) impl_##name
+// not interested in original functions for zone overload
+inline void InitOrigPointers() {}
+
+#endif // MALLOC_UNIXLIKE_OVERLOAD_ENABLED and MALLOC_ZONE_OVERLOAD_ENABLED
+
+void *PREFIX(malloc)(ZONE_ARG size_t size) __THROW
 {
-    return __TBB_internal_malloc(size);
+    return scalable_malloc(size);
 }
 
-void * calloc(size_t num, size_t size) __THROW
+void *PREFIX(calloc)(ZONE_ARG size_t num, size_t size) __THROW
 {
-    return __TBB_internal_calloc(num, size);
+    return scalable_calloc(num, size);
 }
 
-void free(void *object) __THROW
+void PREFIX(free)(ZONE_ARG void *object) __THROW
 {
-    __TBB_internal_free(object);
+    InitOrigPointers();
+    __TBB_malloc_safer_free(object, (void (*)(void*))orig_free);
 }
 
-void * realloc(void* ptr, size_t sz) __THROW
+void *PREFIX(realloc)(ZONE_ARG void* ptr, size_t sz) __THROW
 {
-    return __TBB_internal_realloc(ptr, sz);
-}
-
-int posix_memalign(void **memptr, size_t alignment, size_t size) __THROW
-{
-    return __TBB_internal_posix_memalign(memptr, alignment, size);
+    InitOrigPointers();
+    return __TBB_malloc_safer_realloc(ptr, sz, orig_realloc);
 }
 
 /* The older *NIX interface for aligned allocations;
    it's formally substituted by posix_memalign and deprecated,
    so we do not expect it to cause cyclic dependency with C RTL. */
-void * memalign(size_t alignment, size_t size)  __THROW
+void *PREFIX(memalign)(ZONE_ARG size_t alignment, size_t size) __THROW
 {
     return scalable_aligned_malloc(size, alignment);
 }
 
 /* valloc allocates memory aligned on a page boundary */
-void * valloc(size_t size) __THROW
+void *PREFIX(valloc)(ZONE_ARG size_t size) __THROW
 {
     if (! memoryPageSize) initPageSize();
 
     return scalable_aligned_malloc(size, memoryPageSize);
 }
 
+#undef ZONE_ARG
+#undef PREFIX
+
+#if MALLOC_UNIXLIKE_OVERLOAD_ENABLED
+
+// match prototype from system headers
+#if __ANDROID__
+size_t malloc_usable_size(const void *ptr) __THROW
+#else
+size_t malloc_usable_size(void *ptr) __THROW
+#endif
+{
+    InitOrigPointers();
+    return __TBB_malloc_safer_msize(const_cast<void*>(ptr), (size_t (*)(void*))orig_msize);
+}
+
+int posix_memalign(void **memptr, size_t alignment, size_t size) __THROW
+{
+    return scalable_posix_memalign(memptr, alignment, size);
+}
+
 /* pvalloc allocates smallest set of complete pages which can hold
    the requested number of bytes. Result is aligned on page boundary. */
-void * pvalloc(size_t size) __THROW
+void *pvalloc(size_t size) __THROW
 {
     if (! memoryPageSize) initPageSize();
-    // align size up to the page size
-    size = ((size-1) | (memoryPageSize-1)) + 1;
+    // align size up to the page size,
+    // pvalloc(0) returns 1 page, see man libmpatrol
+    size = size? ((size-1) | (memoryPageSize-1)) + 1 : memoryPageSize;
 
     return scalable_aligned_malloc(size, memoryPageSize);
 }
@@ -139,34 +185,45 @@ int mallopt(int /*param*/, int /*value*/) __THROW
     return 1;
 }
 
-#if !__ANDROID__
-// Those non-standart functions are exported by GLIBC, and might be used
-// in conjunction with standart malloc/free, so we must ovberload them.
-// Bionic doesn't have them. Not removing from the linker scripts,
-// as absent entry points are ignored by the linker.
-void *__libc_malloc(size_t size) __attribute__ ((alias ("malloc")));
-void *__libc_realloc(void *ptr, size_t size) __attribute__ ((alias ("realloc")));
-void *__libc_calloc(size_t num, size_t size) __attribute__ ((alias ("calloc")));
-void __libc_free(void *ptr) __attribute__ ((alias ("free")));
-void *__libc_memalign(size_t alignment, size_t size) __attribute__ ((alias ("memalign")));
-void *__libc_pvalloc(size_t size) __attribute__ ((alias ("pvalloc")));
-void *__libc_valloc(size_t size) __attribute__ ((alias ("valloc")));
-#endif
-
-} /* extern "C" */
-
-#if __linux__
-#include <malloc.h>
-#include <string.h> // for memset
-
-extern "C" struct mallinfo mallinfo() __THROW
+struct mallinfo mallinfo() __THROW
 {
     struct mallinfo m;
     memset(&m, 0, sizeof(struct mallinfo));
 
     return m;
 }
-#endif /* __linux__ */
+
+#if __ANDROID__
+// Android doesn't have malloc_usable_size, provide it to be compatible
+// with Linux, in addition overload dlmalloc_usable_size() that presented
+// under Android.
+size_t dlmalloc_usable_size(const void *ptr) __attribute__ ((alias ("malloc_usable_size")));
+#else // __ANDROID__
+// Those non-standard functions are exported by GLIBC, and might be used
+// in conjunction with standard malloc/free, so we must ovberload them.
+// Bionic doesn't have them. Not removing from the linker scripts,
+// as absent entry points are ignored by the linker.
+void *__libc_malloc(size_t size) __attribute__ ((alias ("malloc")));
+void *__libc_calloc(size_t num, size_t size) __attribute__ ((alias ("calloc")));
+void *__libc_memalign(size_t alignment, size_t size) __attribute__ ((alias ("memalign")));
+void *__libc_pvalloc(size_t size) __attribute__ ((alias ("pvalloc")));
+void *__libc_valloc(size_t size) __attribute__ ((alias ("valloc")));
+
+// call original __libc_* to support naive replacement of free via __libc_free etc
+void __libc_free(void *ptr)
+{
+    InitOrigPointers();
+    __TBB_malloc_safer_free(ptr, (void (*)(void*))orig_libc_free);
+}
+
+void *__libc_realloc(void *ptr, size_t size)
+{
+    InitOrigPointers();
+    return __TBB_malloc_safer_realloc(ptr, size, orig_libc_realloc);
+}
+#endif // !__ANDROID__
+
+} /* extern "C" */
 
 /*** replacements for global operators new and delete ***/
 
@@ -189,10 +246,12 @@ void* operator new[](size_t sz) throw (std::bad_alloc) {
     return res;
 }
 void operator delete(void* ptr) throw() {
-    scalable_free(ptr);
+    InitOrigPointers();
+    __TBB_malloc_safer_free(ptr, (void (*)(void*))orig_free);
 }
 void operator delete[](void* ptr) throw() {
-    scalable_free(ptr);
+    InitOrigPointers();
+    __TBB_malloc_safer_free(ptr, (void (*)(void*))orig_free);
 }
 void* operator new(size_t sz, const std::nothrow_t&) throw() {
     return scalable_malloc(sz);
@@ -201,13 +260,16 @@ void* operator new[](std::size_t sz, const std::nothrow_t&) throw() {
     return scalable_malloc(sz);
 }
 void operator delete(void* ptr, const std::nothrow_t&) throw() {
-    scalable_free(ptr);
+    InitOrigPointers();
+    __TBB_malloc_safer_free(ptr, (void (*)(void*))orig_free);
 }
 void operator delete[](void* ptr, const std::nothrow_t&) throw() {
-    scalable_free(ptr);
+    InitOrigPointers();
+    __TBB_malloc_safer_free(ptr, (void (*)(void*))orig_free);
 }
 
 #endif /* MALLOC_UNIXLIKE_OVERLOAD_ENABLED */
+#endif /* MALLOC_UNIXLIKE_OVERLOAD_ENABLED || MALLOC_ZONE_OVERLOAD_ENABLED */
 
 
 #ifdef _WIN32
@@ -217,10 +279,11 @@ void operator delete[](void* ptr, const std::nothrow_t&) throw() {
 
 #include <stdio.h>
 #include "tbb_function_replacement.h"
+#include "shared_utils.h"
 
-void safer_scalable_free2( void *ptr)
+void __TBB_malloc_safer_delete( void *ptr)
 {
-    safer_scalable_free( ptr, NULL );
+    __TBB_malloc_safer_free( ptr, NULL );
 }
 
 void* safer_aligned_malloc( size_t size, size_t alignment )
@@ -237,33 +300,33 @@ void* safer_expand( void *, size_t )
 
 #define __TBB_ORIG_ALLOCATOR_REPLACEMENT_WRAPPER(CRTLIB)                                        \
 void (*orig_free_##CRTLIB)(void*);                                                              \
-void safer_scalable_free_##CRTLIB(void *ptr)                                                    \
+void __TBB_malloc_safer_free_##CRTLIB(void *ptr)                                                \
 {                                                                                               \
-    safer_scalable_free( ptr, orig_free_##CRTLIB );                                             \
+    __TBB_malloc_safer_free( ptr, orig_free_##CRTLIB );                                         \
 }                                                                                               \
                                                                                                 \
 size_t (*orig_msize_##CRTLIB)(void*);                                                           \
-size_t safer_scalable_msize_##CRTLIB(void *ptr)                                                 \
+size_t __TBB_malloc_safer_msize_##CRTLIB(void *ptr)                                             \
 {                                                                                               \
-    return safer_scalable_msize( ptr, orig_msize_##CRTLIB );                                    \
+    return __TBB_malloc_safer_msize( ptr, orig_msize_##CRTLIB );                                \
 }                                                                                               \
                                                                                                 \
 size_t (*orig_aligned_msize_##CRTLIB)(void*, size_t, size_t);                                   \
-size_t safer_scalable_aligned_msize_##CRTLIB( void *ptr, size_t alignment, size_t offset)       \
+size_t __TBB_malloc_safer_aligned_msize_##CRTLIB( void *ptr, size_t alignment, size_t offset)   \
 {                                                                                               \
-    return safer_scalable_aligned_msize( ptr, alignment, offset, orig_aligned_msize_##CRTLIB ); \
+    return __TBB_malloc_safer_aligned_msize( ptr, alignment, offset, orig_aligned_msize_##CRTLIB ); \
 }                                                                                               \
                                                                                                 \
-void* safer_scalable_realloc_##CRTLIB( void *ptr, size_t size )                                 \
+void* __TBB_malloc_safer_realloc_##CRTLIB( void *ptr, size_t size )                             \
 {                                                                                               \
     orig_ptrs func_ptrs = {orig_free_##CRTLIB, orig_msize_##CRTLIB};                            \
-    return safer_scalable_realloc( ptr, size, &func_ptrs );                                     \
+    return __TBB_malloc_safer_realloc( ptr, size, &func_ptrs );                                 \
 }                                                                                               \
                                                                                                 \
-void* safer_scalable_aligned_realloc_##CRTLIB( void *ptr, size_t size, size_t aligment )        \
+void* __TBB_malloc_safer_aligned_realloc_##CRTLIB( void *ptr, size_t size, size_t aligment )    \
 {                                                                                               \
     orig_ptrs func_ptrs = {orig_free_##CRTLIB, orig_msize_##CRTLIB};                            \
-    return safer_scalable_aligned_realloc( ptr, size, aligment, &func_ptrs );                   \
+    return __TBB_malloc_safer_aligned_realloc( ptr, size, aligment, &func_ptrs );               \
 }
 
 // limit is 30 bytes/60 symbols per line
@@ -293,32 +356,32 @@ const char* known_bytecodes[] = {
 
 #if _WIN64
 #define __TBB_ORIG_ALLOCATOR_REPLACEMENT_CALL(CRT_VER)\
-    ReplaceFunctionWithStore( #CRT_VER "d.dll", "free",  (FUNCPTR)safer_scalable_free_ ## CRT_VER ## d,  known_bytecodes, (FUNCPTR*)&orig_free_ ## CRT_VER ## d );  \
-    ReplaceFunctionWithStore( #CRT_VER  ".dll", "free",  (FUNCPTR)safer_scalable_free_ ## CRT_VER,       known_bytecodes, (FUNCPTR*)&orig_free_ ## CRT_VER );       \
-    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_msize",(FUNCPTR)safer_scalable_msize_ ## CRT_VER ## d, known_bytecodes, (FUNCPTR*)&orig_msize_ ## CRT_VER ## d ); \
-    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_msize",(FUNCPTR)safer_scalable_msize_ ## CRT_VER,      known_bytecodes, (FUNCPTR*)&orig_msize_ ## CRT_VER );      \
-    ReplaceFunctionWithStore( #CRT_VER "d.dll", "realloc",         (FUNCPTR)safer_scalable_realloc_ ## CRT_VER ## d,         0, NULL); \
-    ReplaceFunctionWithStore( #CRT_VER  ".dll", "realloc",         (FUNCPTR)safer_scalable_realloc_ ## CRT_VER,              0, NULL); \
-    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_aligned_free",   (FUNCPTR)safer_scalable_free_ ## CRT_VER ## d,            0, NULL); \
-    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_aligned_free",   (FUNCPTR)safer_scalable_free_ ## CRT_VER,                 0, NULL); \
-    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_aligned_realloc",(FUNCPTR)safer_scalable_aligned_realloc_ ## CRT_VER ## d, 0, NULL); \
-    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_aligned_realloc",(FUNCPTR)safer_scalable_aligned_realloc_ ## CRT_VER,      0, NULL); \
-    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_aligned_msize",(FUNCPTR)safer_scalable_aligned_msize_ ## CRT_VER ## d, known_bytecodes, (FUNCPTR*)&orig_aligned_msize_ ## CRT_VER ## d ); \
-    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_aligned_msize",(FUNCPTR)safer_scalable_aligned_msize_ ## CRT_VER,      known_bytecodes, (FUNCPTR*)&orig_aligned_msize_ ## CRT_VER );
+    ReplaceFunctionWithStore( #CRT_VER "d.dll", "free",  (FUNCPTR)__TBB_malloc_safer_free_ ## CRT_VER ## d,  known_bytecodes, (FUNCPTR*)&orig_free_ ## CRT_VER ## d );  \
+    ReplaceFunctionWithStore( #CRT_VER  ".dll", "free",  (FUNCPTR)__TBB_malloc_safer_free_ ## CRT_VER,       known_bytecodes, (FUNCPTR*)&orig_free_ ## CRT_VER );       \
+    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_msize",(FUNCPTR)__TBB_malloc_safer_msize_ ## CRT_VER ## d, known_bytecodes, (FUNCPTR*)&orig_msize_ ## CRT_VER ## d ); \
+    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_msize",(FUNCPTR)__TBB_malloc_safer_msize_ ## CRT_VER,      known_bytecodes, (FUNCPTR*)&orig_msize_ ## CRT_VER );      \
+    ReplaceFunctionWithStore( #CRT_VER "d.dll", "realloc",         (FUNCPTR)__TBB_malloc_safer_realloc_ ## CRT_VER ## d,         0, NULL); \
+    ReplaceFunctionWithStore( #CRT_VER  ".dll", "realloc",         (FUNCPTR)__TBB_malloc_safer_realloc_ ## CRT_VER,              0, NULL); \
+    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_aligned_free",   (FUNCPTR)__TBB_malloc_safer_free_ ## CRT_VER ## d,            0, NULL); \
+    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_aligned_free",   (FUNCPTR)__TBB_malloc_safer_free_ ## CRT_VER,                 0, NULL); \
+    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_aligned_realloc",(FUNCPTR)__TBB_malloc_safer_aligned_realloc_ ## CRT_VER ## d, 0, NULL); \
+    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_aligned_realloc",(FUNCPTR)__TBB_malloc_safer_aligned_realloc_ ## CRT_VER,      0, NULL); \
+    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_aligned_msize",(FUNCPTR)__TBB_malloc_safer_aligned_msize_ ## CRT_VER ## d, known_bytecodes, (FUNCPTR*)&orig_aligned_msize_ ## CRT_VER ## d ); \
+    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_aligned_msize",(FUNCPTR)__TBB_malloc_safer_aligned_msize_ ## CRT_VER,      known_bytecodes, (FUNCPTR*)&orig_aligned_msize_ ## CRT_VER );
 #else
 #define __TBB_ORIG_ALLOCATOR_REPLACEMENT_CALL(CRT_VER)\
-    ReplaceFunctionWithStore( #CRT_VER "d.dll", "free",  (FUNCPTR)safer_scalable_free_ ## CRT_VER ## d,  known_bytecodes, (FUNCPTR*)&orig_free_ ## CRT_VER ## d );  \
-    ReplaceFunctionWithStore( #CRT_VER  ".dll", "free",  (FUNCPTR)safer_scalable_free_ ## CRT_VER,       known_bytecodes, (FUNCPTR*)&orig_free_ ## CRT_VER );       \
-    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_msize",(FUNCPTR)safer_scalable_msize_ ## CRT_VER ## d, known_bytecodes, (FUNCPTR*)&orig_msize_ ## CRT_VER ## d ); \
-    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_msize",(FUNCPTR)safer_scalable_msize_ ## CRT_VER,      known_bytecodes, (FUNCPTR*)&orig_msize_ ## CRT_VER );      \
-    ReplaceFunctionWithStore( #CRT_VER "d.dll", "realloc",         (FUNCPTR)safer_scalable_realloc_ ## CRT_VER ## d,         0, NULL); \
-    ReplaceFunctionWithStore( #CRT_VER  ".dll", "realloc",         (FUNCPTR)safer_scalable_realloc_ ## CRT_VER,              0, NULL); \
-    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_aligned_free",   (FUNCPTR)safer_scalable_free_ ## CRT_VER ## d,            0, NULL); \
-    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_aligned_free",   (FUNCPTR)safer_scalable_free_ ## CRT_VER,                 0, NULL); \
-    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_aligned_realloc",(FUNCPTR)safer_scalable_aligned_realloc_ ## CRT_VER ## d, 0, NULL); \
-    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_aligned_realloc",(FUNCPTR)safer_scalable_aligned_realloc_ ## CRT_VER,      0, NULL); \
-    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_aligned_msize",(FUNCPTR)safer_scalable_aligned_msize_ ## CRT_VER ## d, known_bytecodes, (FUNCPTR*)&orig_aligned_msize_ ## CRT_VER ## d ); \
-    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_aligned_msize",(FUNCPTR)safer_scalable_aligned_msize_ ## CRT_VER,      known_bytecodes, (FUNCPTR*)&orig_aligned_msize_ ## CRT_VER );
+    ReplaceFunctionWithStore( #CRT_VER "d.dll", "free",  (FUNCPTR)__TBB_malloc_safer_free_ ## CRT_VER ## d,  known_bytecodes, (FUNCPTR*)&orig_free_ ## CRT_VER ## d );  \
+    ReplaceFunctionWithStore( #CRT_VER  ".dll", "free",  (FUNCPTR)__TBB_malloc_safer_free_ ## CRT_VER,       known_bytecodes, (FUNCPTR*)&orig_free_ ## CRT_VER );       \
+    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_msize",(FUNCPTR)__TBB_malloc_safer_msize_ ## CRT_VER ## d, known_bytecodes, (FUNCPTR*)&orig_msize_ ## CRT_VER ## d ); \
+    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_msize",(FUNCPTR)__TBB_malloc_safer_msize_ ## CRT_VER,      known_bytecodes, (FUNCPTR*)&orig_msize_ ## CRT_VER );      \
+    ReplaceFunctionWithStore( #CRT_VER "d.dll", "realloc",         (FUNCPTR)__TBB_malloc_safer_realloc_ ## CRT_VER ## d,         0, NULL); \
+    ReplaceFunctionWithStore( #CRT_VER  ".dll", "realloc",         (FUNCPTR)__TBB_malloc_safer_realloc_ ## CRT_VER,              0, NULL); \
+    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_aligned_free",   (FUNCPTR)__TBB_malloc_safer_free_ ## CRT_VER ## d,            0, NULL); \
+    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_aligned_free",   (FUNCPTR)__TBB_malloc_safer_free_ ## CRT_VER,                 0, NULL); \
+    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_aligned_realloc",(FUNCPTR)__TBB_malloc_safer_aligned_realloc_ ## CRT_VER ## d, 0, NULL); \
+    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_aligned_realloc",(FUNCPTR)__TBB_malloc_safer_aligned_realloc_ ## CRT_VER,      0, NULL); \
+    ReplaceFunctionWithStore( #CRT_VER "d.dll", "_aligned_msize",(FUNCPTR)__TBB_malloc_safer_aligned_msize_ ## CRT_VER ## d, known_bytecodes, (FUNCPTR*)&orig_aligned_msize_ ## CRT_VER ## d ); \
+    ReplaceFunctionWithStore( #CRT_VER  ".dll", "_aligned_msize",(FUNCPTR)__TBB_malloc_safer_aligned_msize_ ## CRT_VER,      known_bytecodes, (FUNCPTR*)&orig_aligned_msize_ ## CRT_VER );
 #endif
 
 __TBB_ORIG_ALLOCATOR_REPLACEMENT_WRAPPER(msvcr70d);
@@ -357,14 +420,14 @@ void* operator_new_arr(size_t sz) throw (std::bad_alloc) {
     return res;
 }
 void operator_delete(void* ptr) throw() {
-    safer_scalable_free2(ptr);
+    __TBB_malloc_safer_delete(ptr);
 }
 #if _MSC_VER && !defined(__INTEL_COMPILER)
 #pragma warning( pop )
 #endif
 
 void operator_delete_arr(void* ptr) throw() {
-    safer_scalable_free2(ptr);
+    __TBB_malloc_safer_delete(ptr);
 }
 void* operator_new_t(size_t sz, const std::nothrow_t&) throw() {
     return scalable_malloc(sz);
@@ -373,10 +436,10 @@ void* operator_new_arr_t(std::size_t sz, const std::nothrow_t&) throw() {
     return scalable_malloc(sz);
 }
 void operator_delete_t(void* ptr, const std::nothrow_t&) throw() {
-    safer_scalable_free2(ptr);
+    __TBB_malloc_safer_delete(ptr);
 }
 void operator_delete_arr_t(void* ptr, const std::nothrow_t&) throw() {
-    safer_scalable_free2(ptr);
+    __TBB_malloc_safer_delete(ptr);
 }
 
 const char* modules_to_replace[] = {
@@ -466,8 +529,6 @@ void ReplaceFunctionWithStore( const wchar_t *dllName, const char *funcName, FUN
 
 void doMallocReplacement()
 {
-    int i,j;
-
     // Replace functions and keep backup of original code (separate for each runtime)
     __TBB_ORIG_ALLOCATOR_REPLACEMENT_CALL(msvcr70)
     __TBB_ORIG_ALLOCATOR_REPLACEMENT_CALL(msvcr71)
@@ -478,10 +539,8 @@ void doMallocReplacement()
     __TBB_ORIG_ALLOCATOR_REPLACEMENT_CALL(msvcr120)
 
     // Replace functions without storing original code
-    int modules_to_replace_count = sizeof(modules_to_replace) / sizeof(modules_to_replace[0]);
-    int routines_to_replace_count = sizeof(routines_to_replace) / sizeof(routines_to_replace[0]);
-    for ( j=0; j<modules_to_replace_count; j++ )
-        for (i = 0; i < routines_to_replace_count; i++)
+    for ( size_t j=0; j < arrayLength(modules_to_replace); j++ )
+        for (size_t i = 0; i < arrayLength(routines_to_replace); i++)
         {
 #if !_WIN64
             // in Microsoft* Visual Studio* 2012 and 2013 32-bit operator delete consists of 2 bytes only: short jump to free(ptr);
