@@ -1,40 +1,34 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 const unsigned MByte = 1024*1024;
 bool __tbb_test_errno = false;
 
+#define __STDC_LIMIT_MACROS 1 // to get SIZE_MAX from stdint.h
+
 #include "tbb/tbb_config.h"
 
-#if __TBB_WIN8UI_SUPPORT	
+#if __TBB_WIN8UI_SUPPORT
 // testing allocator itself not iterfaces
 // so we can use desktop functions
-#define _CRT_USE_WINAPI_FAMILY_DESKTOP_APP 1
+#define _CRT_USE_WINAPI_FAMILY_DESKTOP_APP !_M_ARM
 #define HARNESS_NO_PARSE_COMMAND_LINE 1
 #include "harness.h"
 // FIXME: fix the test to support New Windows *8 Store Apps mode.
@@ -43,11 +37,10 @@ int TestMain() {
 }
 #else /* __TBB_WIN8UI_SUPPORT	 */
 
+#if _WIN32 || _WIN64
 /* _WIN32_WINNT should be defined at the very beginning,
    because other headers might include <windows.h>
 */
-
-#if _WIN32 || _WIN64
 #undef _WIN32_WINNT
 #define _WIN32_WINNT 0x0501
 #include "tbb/machine/windows_api.h"
@@ -82,36 +75,45 @@ void limitMem( size_t limit )
     }
 }
 // Do not test errno with static VC runtime
-#else
+#else // _WIN32 || _WIN64
 #include <sys/resource.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
 #include <sys/types.h>  // uint64_t on FreeBSD, needed for rlim_t
+#include <stdint.h>     // SIZE_MAX
 #include "harness_report.h"
 
 void limitMem( size_t limit )
 {
     rlimit rlim;
-    rlim.rlim_cur = limit? limit*MByte : (rlim_t)RLIM_INFINITY;
-    rlim.rlim_max = (rlim_t)RLIM_INFINITY;
-    int ret = setrlimit(RLIMIT_AS,&rlim);
+    int ret = getrlimit(RLIMIT_AS,&rlim);
+    if (0 != ret) {
+        REPORT("getrlimit() returned an error: errno %d\n", errno);
+        exit(1);
+    }
+    if (rlim.rlim_max==(rlim_t)RLIM_INFINITY)
+        rlim.rlim_cur = (limit > 0) ? limit*MByte : rlim.rlim_max;
+    else rlim.rlim_cur = (limit > 0 && limit<rlim.rlim_max) ? limit*MByte : rlim.rlim_max;
+    ret = setrlimit(RLIMIT_AS,&rlim);
     if (0 != ret) {
         REPORT("Can't set limits: errno %d\n", errno);
         exit(1);
     }
 }
-#endif
+#endif  // _WIN32 || _WIN64
 
 #define ASSERT_ERRNO(cond, msg)  ASSERT( !__tbb_test_errno || (cond), msg )
 #define CHECK_ERRNO(cond) (__tbb_test_errno && (cond))
 
 #include <time.h>
 #include <errno.h>
+#include <limits.h> // for CHAR_BIT
 #define __TBB_NO_IMPLICIT_LINKAGE 1
 #include "tbb/scalable_allocator.h"
 
 #define HARNESS_CUSTOM_MAIN 1
+#define HARNESS_TBBMALLOC_THREAD_SHUTDOWN 1
 #include "harness.h"
 #include "harness_barrier.h"
 #if !__TBB_SOURCE_DIRECTLY_INCLUDED
@@ -141,8 +143,6 @@ extern "C" void *__cdecl _aligned_malloc(size_t,size_t);
     #pragma warning (pop)
 #endif
 
-const size_t COUNT_ELEM_CALLOC = 2;
-const int COUNT_TESTS = 1000;
 const int COUNT_ELEM = 25000;
 const size_t MAX_SIZE = 1000;
 const int COUNTEXPERIMENT = 10000;
@@ -183,8 +183,10 @@ TestAlignedRealloc* Raligned_realloc;
 bool error_occurred = false;
 
 #if __APPLE__
-// Tests that use the variable are skipped on OS X*
+// Tests that use the variables are skipped on OS X*
 #else
+const size_t COUNT_ELEM_CALLOC = 2;
+const int COUNT_TESTS = 1000;
 static bool perProcessLimits = true;
 #endif
 
@@ -306,6 +308,57 @@ void ReallocParam()
         Trealloc(bufs[j], 0);
 }
 
+void CheckArgumentsOverflow()
+{
+    void *p;
+    const size_t params[] = {SIZE_MAX, SIZE_MAX-16};
+
+    for (unsigned i=0; i<Harness::array_length(params); i++) {
+        p = Tmalloc(params[i]);
+        ASSERT(!p, NULL);
+        ASSERT_ERRNO(errno==ENOMEM, NULL);
+        p = Trealloc(NULL, params[i]);
+        ASSERT(!p, NULL);
+        ASSERT_ERRNO(errno==ENOMEM, NULL);
+        p = Tcalloc(1, params[i]);
+        ASSERT(!p, NULL);
+        ASSERT_ERRNO(errno==ENOMEM, NULL);
+        p = Tcalloc(params[i], 1);
+        ASSERT(!p, NULL);
+        ASSERT_ERRNO(errno==ENOMEM, NULL);
+    }
+    const size_t max_alignment = size_t(1) << (sizeof(size_t)*CHAR_BIT - 1);
+    if (Rposix_memalign) {
+        int ret = Rposix_memalign(&p, max_alignment, ~max_alignment);
+        ASSERT(ret == ENOMEM, NULL);
+        for (unsigned i=0; i<Harness::array_length(params); i++) {
+            ret = Rposix_memalign(&p, max_alignment, params[i]);
+            ASSERT(ret == ENOMEM, NULL);
+            ret = Rposix_memalign(&p, sizeof(void*), params[i]);
+            ASSERT(ret == ENOMEM, NULL);
+        }
+    }
+    if (Raligned_malloc) {
+        p = Raligned_malloc(~max_alignment, max_alignment);
+        ASSERT(!p, NULL);
+        for (unsigned i=0; i<Harness::array_length(params); i++) {
+            p = Raligned_malloc(params[i], max_alignment);
+            ASSERT(!p, NULL);
+            ASSERT_ERRNO(errno==ENOMEM, NULL);
+            p = Raligned_malloc(params[i], sizeof(void*));
+            ASSERT(!p, NULL);
+            ASSERT_ERRNO(errno==ENOMEM, NULL);
+        }
+    }
+
+    p = Tcalloc(SIZE_MAX/2-16, SIZE_MAX/2-16);
+    ASSERT(!p, NULL);
+    ASSERT_ERRNO(errno==ENOMEM, NULL);
+    p = Tcalloc(SIZE_MAX/2, SIZE_MAX/2);
+    ASSERT(!p, NULL);
+    ASSERT_ERRNO(errno==ENOMEM, NULL);
+}
+
 HARNESS_EXPORT
 int main(int argc, char* argv[]) {
     argC=argc;
@@ -324,9 +377,17 @@ int main(int argc, char* argv[]) {
     for (int i=1; i< argc; i++) {
         if (strcmp((char*)*(argv+i),"-s")==0)
         {
+#if __INTEL_COMPILER == 1400 && __linux__
+            // Workaround for Intel(R) C++ Compiler XE, version 14.0.0.080:
+            // unable to call setSystemAllocs() in such configuration.
+            REPORT("Known issue: Standard allocator testing is not supported.\n");
+            REPORT( "skip\n" );
+            return 0;
+#else
             setSystemAllocs();
             argC--;
             break;
+#endif
         }
     }
 
@@ -376,12 +437,20 @@ int main(int argc, char* argv[]) {
     __tbb_test_errno = true;
 #endif // _MSC_VER
 
+    CheckArgumentsOverflow();
     for( int p=MaxThread; p>=MinThread; --p ) {
         REMARK("testing with %d threads\n", p );
-        Harness::SpinBarrier *barrier = new Harness::SpinBarrier(p);
-        NativeParallelFor( p, RoundRobin(p, barrier, Verbose) );
-        delete barrier;
+        for (int limit=0; limit<2; limit++) {
+            int ret = scalable_allocation_mode(TBBMALLOC_SET_SOFT_HEAP_LIMIT,
+                                               16*1024*limit);
+            ASSERT(ret==TBBMALLOC_OK, NULL);
+            Harness::SpinBarrier *barrier = new Harness::SpinBarrier(p);
+            NativeParallelFor( p, RoundRobin(p, barrier, Verbose) );
+            delete barrier;
+        }
     }
+    int ret = scalable_allocation_mode(TBBMALLOC_SET_SOFT_HEAP_LIMIT, 0);
+    ASSERT(ret==TBBMALLOC_OK, NULL);
     if( !error_occurred )
         REPORT("done\n");
     return 0;
@@ -928,7 +997,7 @@ void CMemTest::Free_NULL()
     for (int i=0; i<COUNTEXPERIMENT; i++)
     {
         Tfree(NULL);
-        if (errno != 0)
+        if (CHECK_ERRNO(errno))
         {
             CountErrors++;
             if (ShouldReportError()) REPORT("error is found by a call free with parameter NULL\n");
@@ -1018,17 +1087,14 @@ void CMemTest::RunAllTests(int total_threads)
     if (Raligned_realloc)
         InvariantDataRealloc(/*aligned=*/true);
     TestAlignedParameters();
+    UniquePointer();
+    AddrArifm();
 #if __APPLE__
     REPORT("Known issue: some tests are skipped on OS X*\n");
 #else
-    UniquePointer();
-    AddrArifm();
-#if !__TBB_MIC_NATIVE
     NULLReturn(1*MByte,100*MByte,total_threads);
-#endif
 #endif
     if (FullLog) REPORT("Tests for %d threads ended\n", total_threads);
 }
 
 #endif /* __TBB_WIN8UI_SUPPORT	 */
-
