@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2013 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #include "harness_task.h"
@@ -84,12 +76,12 @@ tbb::atomic<int> EnqueuedTask::nOrderedPairs;
 
 const int nTracks = 10;
 static int TaskTracks[nTracks];
-const int stall_threshold = 100000;
+const int stall_threshold = 1000000; // 1 sec
 
 void TimedYield( double pause_time ) {
     tbb::tick_count start = tbb::tick_count::now();
     while( (tbb::tick_count::now()-start).seconds() < pause_time )
-        __TBB_Yield();
+        tbb::this_tbb_thread::sleep(tbb::tick_count::interval_t(pause_time));
 }
 
 class ProgressMonitor {
@@ -119,8 +111,8 @@ public:
                 stall_count=0;
             else {
                 ++stall_count;
-                // no progress for at least 0.1 s; consider it dead.
-                ASSERT(stall_count < stall_threshold, "no progress on enqueued tasks; deadlock, or the machine is oversubscribed?");
+                // no progress; consider it dead.
+                ASSERT(stall_count < stall_threshold, "no progress on enqueued tasks; deadlock, or the machine is heavily oversubscribed?");
             }
             if( progress_mask==all_progressed || progress_mask^last_progress_mask ) {
                 uneven_progress_count = 0;
@@ -128,7 +120,9 @@ public:
             }
             else if ( overall_progress > 2 ) {
                 ++uneven_progress_count;
-                ASSERT(uneven_progress_count < 5, "some enqueued tasks seem stalling; no simultaneous progress?");
+                // The threshold of 32 is 4x bigger than what was observed on a 8-core machine with oversubscription.
+                ASSERT_WARNING(uneven_progress_count < 32,
+                    "some enqueued tasks seem stalling; no simultaneous progress, or the machine is oversubscribed? Investigate if repeated");
             }
         } while( !completed );
     }
@@ -216,6 +210,7 @@ public:
     void Execute() {
         while ( !CanStart )
             __TBB_Yield();
+        Harness::Sleep(10); // increases probability of the bug
         tbb::task::enqueue( *new( tbb::task::allocate_root() ) CarrierTask(m_taskToSpawn) );
     }
 };
@@ -253,10 +248,10 @@ void TestCascadedEnqueue () {
             }
             for ( j = 0; j < numThreads; ++j )
                 Finished[j] = false;
-            REMARK("%02d threads; Iteration %03d\r", numThreads, i);
+            REMARK("\r%02d threads; Iteration %03d", numThreads, i);
         }
     }
-    REMARK( "                                 \r" );
+    REMARK( "\r                                 \r" );
 }
 
 class DummyTask : public tbb::task {
@@ -323,7 +318,8 @@ void TestDequeueByMaster () {
 #include "tbb/parallel_for.h"
 
 static const int NUM_TASKS    = 4;
-static const size_t NUM_REPEATS = 100000;
+static const size_t NUM_REPEATS = TBB_USE_DEBUG ? 50000 : 100000;
+static tbb::task_group_context persistent_context(tbb::task_group_context::isolated);
 
 struct Functor : NoAssign
 {
@@ -332,7 +328,9 @@ struct Functor : NoAssign
     void operator()(const tbb::blocked_range<int>& r) const 
     {
         ASSERT(r.size() == 1, NULL);
-        tbb::task *t = new(tbb::task::allocate_root()) tbb::empty_task();
+        // allocate_root() uses current context of parallel_for which is destroyed when it finishes.
+        // But enqueued tasks can outlive parallel_for execution. Thus, use a persistent context.
+        tbb::task *t = new(tbb::task::allocate_root(persistent_context)) tbb::empty_task();
         tbb::task::enqueue(*t); // ensure no missing wake-ups
         my_barrier.timed_wait(10, "Attention: poorly reproducible event, if seen stress testing required" );
     }
